@@ -2,9 +2,10 @@ from __future__ import annotations
 
 import datetime as dt
 import time
+from collections.abc import AsyncGenerator, AsyncIterator, Iterator
 from http.cookies import SimpleCookie
 from pathlib import Path
-from typing import AsyncIterator, Iterator
+from typing import Any
 
 import anyio
 import pytest
@@ -12,14 +13,8 @@ import pytest
 from starlette import status
 from starlette.background import BackgroundTask
 from starlette.datastructures import Headers
-from starlette.requests import Request
-from starlette.responses import (
-    FileResponse,
-    JSONResponse,
-    RedirectResponse,
-    Response,
-    StreamingResponse,
-)
+from starlette.requests import ClientDisconnect, Request
+from starlette.responses import FileResponse, JSONResponse, RedirectResponse, Response, StreamingResponse
 from starlette.testclient import TestClient
 from starlette.types import Message, Receive, Scope, Send
 from tests.types import TestClientFactory
@@ -89,13 +84,13 @@ def test_redirect_response_content_length_header(
 ) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         if scope["path"] == "/":
-            response = Response("hello", media_type="text/plain")  # pragma: nocover
+            response = Response("hello", media_type="text/plain")  # pragma: no cover
         else:
             response = RedirectResponse("/")
         await response(scope, receive, send)
 
     client: TestClient = test_client_factory(app)
-    response = client.request("GET", "/redirect", allow_redirects=False)
+    response = client.request("GET", "/redirect", follow_redirects=False)
     assert response.url == "http://testserver/redirect"
     assert response.headers["content-length"] == "0"
 
@@ -271,13 +266,13 @@ async def test_file_response_on_head_method(tmp_path: Path) -> None:
             assert "content-disposition" in headers
             assert "last-modified" in headers
             assert "etag" in headers
-        elif message["type"] == "http.response.body":
+        elif message["type"] == "http.response.body":  # pragma: no branch
             assert message["body"] == b""
             assert message["more_body"] is False
 
     # Since the TestClient drops the response body on HEAD requests, we need to test
     # this directly.
-    await app({"type": "http", "method": "head"}, receive, send)
+    await app({"type": "http", "method": "head", "headers": [(b"key", b"value")]}, receive, send)
 
 
 def test_file_response_set_media_type(tmp_path: Path, test_client_factory: TestClientFactory) -> None:
@@ -342,6 +337,22 @@ def test_file_response_with_method_warns(tmp_path: Path) -> None:
         FileResponse(path=tmp_path, filename="example.png", method="GET")
 
 
+def test_file_response_with_range_header(tmp_path: Path, test_client_factory: TestClientFactory) -> None:
+    content = b"file content"
+    filename = "hello.txt"
+    path = tmp_path / filename
+    path.write_bytes(content)
+    etag = '"a_non_autogenerated_etag"'
+    app = FileResponse(path=path, filename=filename, headers={"etag": etag})
+    client = test_client_factory(app)
+    response = client.get("/", headers={"range": "bytes=0-4", "if-range": etag})
+    assert response.status_code == status.HTTP_206_PARTIAL_CONTENT
+    assert response.content == content[:5]
+    assert response.headers["etag"] == etag
+    assert response.headers["content-length"] == "5"
+    assert response.headers["content-range"] == f"bytes 0-4/{len(content)}"
+
+
 def test_set_cookie(test_client_factory: TestClientFactory, monkeypatch: pytest.MonkeyPatch) -> None:
     # Mock time used as a reference for `Expires` by stdlib `SimpleCookie`.
     mocked_now = dt.datetime(2037, 1, 22, 12, 0, 0, tzinfo=dt.timezone.utc)
@@ -383,6 +394,18 @@ def test_set_cookie_path_none(test_client_factory: TestClientFactory) -> None:
     assert response.headers["set-cookie"] == "mycookie=myvalue; SameSite=lax"
 
 
+def test_set_cookie_samesite_none(test_client_factory: TestClientFactory) -> None:
+    async def app(scope: Scope, receive: Receive, send: Send) -> None:
+        response = Response("Hello, world!", media_type="text/plain")
+        response.set_cookie("mycookie", "myvalue", samesite=None)
+        await response(scope, receive, send)
+
+    client = test_client_factory(app)
+    response = client.get("/")
+    assert response.text == "Hello, world!"
+    assert response.headers["set-cookie"] == "mycookie=myvalue; Path=/"
+
+
 @pytest.mark.parametrize(
     "expires",
     [
@@ -546,3 +569,233 @@ async def test_streaming_response_stops_if_receiving_http_disconnect() -> None:
     with anyio.move_on_after(1) as cancel_scope:
         await response({}, receive_disconnect, send)
     assert not cancel_scope.cancel_called, "Content streaming should stop itself."
+
+
+@pytest.mark.anyio
+async def test_streaming_response_on_client_disconnects() -> None:
+    chunks = bytearray()
+    streamed = False
+
+    async def receive_disconnect() -> Message:
+        raise NotImplementedError
+
+    async def send(message: Message) -> None:
+        nonlocal streamed
+        if message["type"] == "http.response.body":
+            if not streamed:
+                chunks.extend(message.get("body", b""))
+                streamed = True
+            else:
+                raise OSError
+
+    async def stream_indefinitely() -> AsyncGenerator[bytes, None]:
+        while True:
+            await anyio.sleep(0)
+            yield b"chunk"
+
+    stream = stream_indefinitely()
+    response = StreamingResponse(content=stream)
+
+    with anyio.move_on_after(1) as cancel_scope:
+        with pytest.raises(ClientDisconnect):
+            await response({"asgi": {"spec_version": "2.4"}}, receive_disconnect, send)
+    assert not cancel_scope.cancel_called, "Content streaming should stop itself."
+    assert chunks == b"chunk"
+    await stream.aclose()
+
+
+README = """\
+# BáiZé
+
+Powerful and exquisite WSGI/ASGI framework/toolkit.
+
+The minimize implementation of methods required in the Web framework. No redundant implementation means that you can freely customize functions without considering the conflict with baize's own implementation.
+
+Under the ASGI/WSGI protocol, the interface of the request object and the response object is almost the same, only need to add or delete `await` in the appropriate place. In addition, it should be noted that ASGI supports WebSocket but WSGI does not.
+"""  # noqa: E501
+
+
+@pytest.fixture
+def readme_file(tmp_path: Path) -> Path:
+    filepath = tmp_path / "README.txt"
+    filepath.write_bytes(README.encode("utf8"))
+    return filepath
+
+
+@pytest.fixture
+def file_response_client(readme_file: Path, test_client_factory: TestClientFactory) -> TestClient:
+    return test_client_factory(app=FileResponse(str(readme_file)))
+
+
+def test_file_response_without_range(file_response_client: TestClient) -> None:
+    response = file_response_client.get("/")
+    assert response.status_code == 200
+    assert response.headers["content-length"] == str(len(README.encode("utf8")))
+    assert response.text == README
+
+
+def test_file_response_head(file_response_client: TestClient) -> None:
+    response = file_response_client.head("/")
+    assert response.status_code == 200
+    assert response.headers["content-length"] == str(len(README.encode("utf8")))
+    assert response.content == b""
+
+
+def test_file_response_range(file_response_client: TestClient) -> None:
+    response = file_response_client.get("/", headers={"Range": "bytes=0-100"})
+    assert response.status_code == 206
+    assert response.headers["content-range"] == f"bytes 0-100/{len(README.encode('utf8'))}"
+    assert response.headers["content-length"] == "101"
+    assert response.content == README.encode("utf8")[:101]
+
+
+def test_file_response_range_head(file_response_client: TestClient) -> None:
+    response = file_response_client.head("/", headers={"Range": "bytes=0-100"})
+    assert response.status_code == 206
+    assert response.headers["content-length"] == str(101)
+    assert response.content == b""
+
+
+def test_file_response_range_multi(file_response_client: TestClient) -> None:
+    response = file_response_client.get("/", headers={"Range": "bytes=0-100, 200-300"})
+    assert response.status_code == 206
+    assert response.headers["content-range"].startswith("multipart/byteranges; boundary=")
+    assert response.headers["content-length"] == "439"
+
+
+def test_file_response_range_multi_head(file_response_client: TestClient) -> None:
+    response = file_response_client.head("/", headers={"Range": "bytes=0-100, 200-300"})
+    assert response.status_code == 206
+    assert response.headers["content-length"] == "439"
+    assert response.content == b""
+
+    response = file_response_client.head(
+        "/",
+        headers={"Range": "bytes=200-300", "if-range": response.headers["etag"][:-1]},
+    )
+    assert response.status_code == 200
+    response = file_response_client.head(
+        "/",
+        headers={"Range": "bytes=200-300", "if-range": response.headers["etag"]},
+    )
+    assert response.status_code == 206
+
+
+def test_file_response_range_invalid(file_response_client: TestClient) -> None:
+    response = file_response_client.head("/", headers={"Range": "bytes: 0-1000"})
+    assert response.status_code == 400
+
+
+def test_file_response_range_head_max(file_response_client: TestClient) -> None:
+    response = file_response_client.head("/", headers={"Range": f"bytes=0-{len(README.encode('utf8'))+1}"})
+    assert response.status_code == 206
+
+
+def test_file_response_range_416(file_response_client: TestClient) -> None:
+    response = file_response_client.head("/", headers={"Range": f"bytes={len(README.encode('utf8'))+1}-"})
+    assert response.status_code == 416
+    assert response.headers["Content-Range"] == f"*/{len(README.encode('utf8'))}"
+
+
+def test_file_response_only_support_bytes_range(file_response_client: TestClient) -> None:
+    response = file_response_client.get("/", headers={"Range": "items=0-100"})
+    assert response.status_code == 400
+    assert response.text == "Only support bytes range"
+
+
+def test_file_response_range_must_be_requested(file_response_client: TestClient) -> None:
+    response = file_response_client.get("/", headers={"Range": "bytes="})
+    assert response.status_code == 400
+    assert response.text == "Range header: range must be requested"
+
+
+def test_file_response_start_must_be_less_than_end(file_response_client: TestClient) -> None:
+    response = file_response_client.get("/", headers={"Range": "bytes=100-0"})
+    assert response.status_code == 400
+    assert response.text == "Range header: start must be less than end"
+
+
+def test_file_response_merge_ranges(file_response_client: TestClient) -> None:
+    response = file_response_client.get("/", headers={"Range": "bytes=0-100, 50-200"})
+    assert response.status_code == 206
+    assert response.headers["content-length"] == "201"
+    assert response.headers["content-range"] == f"bytes 0-200/{len(README.encode('utf8'))}"
+
+
+def test_file_response_insert_ranges(file_response_client: TestClient) -> None:
+    response = file_response_client.get("/", headers={"Range": "bytes=100-200, 0-50"})
+
+    assert response.status_code == 206
+    assert response.headers["content-range"].startswith("multipart/byteranges; boundary=")
+    boundary = response.headers["content-range"].split("boundary=")[1]
+    assert response.text.splitlines() == [
+        f"--{boundary}",
+        "Content-Type: text/plain; charset=utf-8",
+        "Content-Range: bytes 0-50/526",
+        "",
+        "# BáiZé",
+        "",
+        "Powerful and exquisite WSGI/ASGI framewo",
+        f"--{boundary}",
+        "Content-Type: text/plain; charset=utf-8",
+        "Content-Range: bytes 100-200/526",
+        "",
+        "ds required in the Web framework. No redundant implementation means that you can freely customize fun",
+        "",
+        f"--{boundary}--",
+    ]
+
+
+@pytest.mark.anyio
+async def test_file_response_multi_small_chunk_size(readme_file: Path) -> None:
+    class SmallChunkSizeFileResponse(FileResponse):
+        chunk_size = 10
+
+    app = SmallChunkSizeFileResponse(path=str(readme_file))
+
+    received_chunks: list[bytes] = []
+    start_message: dict[str, Any] = {}
+
+    async def receive() -> Message:
+        raise NotImplementedError("Should not be called!")
+
+    async def send(message: Message) -> None:
+        if message["type"] == "http.response.start":
+            start_message.update(message)
+        elif message["type"] == "http.response.body":  # pragma: no branch
+            received_chunks.append(message["body"])
+
+    await app({"type": "http", "method": "get", "headers": [(b"range", b"bytes=0-15,20-35,35-50")]}, receive, send)
+    assert start_message["status"] == 206
+
+    headers = Headers(raw=start_message["headers"])
+    assert headers.get("content-type") == "text/plain; charset=utf-8"
+    assert headers.get("accept-ranges") == "bytes"
+    assert "content-length" in headers
+    assert "last-modified" in headers
+    assert "etag" in headers
+    assert headers["content-range"].startswith("multipart/byteranges; boundary=")
+    boundary = headers["content-range"].split("boundary=")[1]
+
+    assert received_chunks == [
+        # Send the part headers.
+        f"--{boundary}\nContent-Type: text/plain; charset=utf-8\nContent-Range: bytes 0-15/526\n\n".encode(),
+        # Send the first chunk (10 bytes).
+        b"# B\xc3\xa1iZ\xc3\xa9\n",
+        # Send the second chunk (6 bytes).
+        b"\nPower",
+        # Send the new line to separate the parts.
+        b"\n",
+        # Send the part headers. We merge the ranges 20-35 and 35-50 into a single part.
+        f"--{boundary}\nContent-Type: text/plain; charset=utf-8\nContent-Range: bytes 20-50/526\n\n".encode(),
+        # Send the first chunk (10 bytes).
+        b"and exquis",
+        # Send the second chunk (10 bytes).
+        b"ite WSGI/A",
+        # Send the third chunk (10 bytes).
+        b"SGI framew",
+        # Send the last chunk (1 byte).
+        b"o",
+        b"\n",
+        f"\n--{boundary}--\n".encode(),
+    ]
